﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace SilverMC
{
    public partial class AcountWindow : Form, IDisposable
    {
        private SilverMCData config;

        public AcountWindow(SilverMCData cf)
        {
            InitializeComponent();
            config = cf;
            ListReload();
            if (config.DefaultAcount != -1 && config.DefaultAcount < config.Acounts.Count)
            {
                comboAcounts.SelectedIndex = config.DefaultAcount;
            }
        }

        private void ListReload()
        {
            comboAcounts.Items.Clear();
            listAcounts.Items.Clear();
            for (int i = 0; i < config.Acounts.Count; i++)
            {
                listAcounts.Items.Add(Utils.DecryptString(config.Acounts[i].UserName, Utils.SOURCE_STRING));
                comboAcounts.Items.Add(Utils.DecryptString(config.Acounts[i].UserName, Utils.SOURCE_STRING));
            }
        }

        private void textUserId_TextChanged(object sender, EventArgs e)
        {
            AddBtnCheck();
        }

        private void textPassword_MaskInputRejected(object sender, MaskInputRejectedEventArgs e)
        {
            AddBtnCheck();
        }

        private void AddBtnCheck()
        {
            if (textUserId.Text != null && textUserId.Text.Length > 0 && textPassword.Text != null && textPassword.Text.Length > 0)
            {
                btnAdd.Enabled = true;
            }
            else
            {
                btnAdd.Enabled = false;
            }
        }

        private void textPassword_TextChanged(object sender, EventArgs e)
        {
            AddBtnCheck();
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            Acount ac = new Acount();
            ac.UserName = Utils.EncryptString(textUserId.Text, Utils.SOURCE_STRING);
            ac.PassWord = Utils.EncryptString(textPassword.Text, Utils.SOURCE_STRING);
            textUserId.Text = "";
            textPassword.Text = "";
            config.Acounts.Add(ac);
            int comboSelect = comboAcounts.SelectedIndex;
            ListReload();
            if (comboAcounts.SelectedIndex == -1 && comboSelect != -1)
            {
                comboAcounts.SelectedIndex = comboSelect;
            }
            else
            {
                btnFinish.Enabled = true;
            }
        }

        private void comboAcounts_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (comboAcounts.SelectedIndex != -1)
            {
                btnFinish.Enabled = true;
            }
            else
            {
                btnFinish.Enabled = false;
            }
        }

        private void btnFinish_Click(object sender, EventArgs e)
        {
            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            config.DefaultAcount = comboAcounts.SelectedIndex;
            this.Hide();
        }

        private void btnDel_Click(object sender, EventArgs e)
        {
            if (listAcounts.SelectedIndex != -1 && comboAcounts.SelectedIndex != listAcounts.SelectedIndex)
            {
                config.Acounts.RemoveAt(listAcounts.SelectedIndex);
                listAcounts.Items.RemoveAt(listAcounts.SelectedIndex);
            }
            else if (comboAcounts.SelectedIndex == listAcounts.SelectedIndex)
            {
                MessageBox.Show("デフォルトのアカウントを削除することはできません", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }
        
    }
}
