﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;

namespace SilverMC
{
    public partial class EditWindow : Form
    {
        private GameProfile result;
        private Versions versions;
        private SilverMCData config;
        private GameProfile gp;
        private string directory;

        public EditWindow(SilverMCData c, GameProfile gd, Versions vs, string dir)
        {
            InitializeComponent();
            versions = vs;
            gp = gd;
            config = c;
            directory = dir;
        }

        private void EditWindow_Load(object sender, EventArgs e)
        {
            if (!(gp.AcountIndex < config.Acounts.Count))
            {
                gp.AcountIndex = -1;
            }
            tbName.Text = gp.DisplayName;
            tbDirectory.Text = "(default)";
            nuWidth.Value = gp.Width;
            nuHeight.Value = gp.Height;
            checkAcount.Checked = gp.AcountIndex != -1;
            checkServer.Checked = gp.JoinServer;
            tbServer.Text = gp.ServerAddress != null ? gp.ServerAddress + ":" + gp.ServerPort : "";
            nuXms.Value = gp.JavaXms;
            nuXmx.Value = gp.JavaXmx;

            cbVersion.Items.Clear();
            for (int i = 0; i < versions.Games.Count; i++)
            {
                cbVersion.Items.Add(versions.Games[i].Id);
            }
            cbAcounts.Items.Clear();
            for (int i = 0; i < config.Acounts.Count; i++)
            {
                cbAcounts.Items.Add(Utils.DecryptString(config.Acounts[i].UserName, Utils.SOURCE_STRING));
            }

            if(gp.AcountIndex != -1) cbAcounts.SelectedIndex = gp.AcountIndex;
            cbVersion.SelectedIndex = cbVersion.Items.IndexOf(gp.Version);
        }

        private void checkAcount_CheckedChanged(object sender, EventArgs e)
        {
            cbAcounts.Enabled = checkAcount.Checked;
        }

        private void checkServer_CheckedChanged(object sender, EventArgs e)
        {
            tbServer.Enabled = checkServer.Checked;
        }

        private void btnFinish_Click(object sender, EventArgs e)
        {
            if (checkServer.Checked)
            {
                if (!(tbServer.Text != null && tbServer.Text.Length > 0))
                {
                    return;
                }
            }

            if ((tbName.Text != null && tbName.Text.Length > 0))
            {

            }
            else
            {
                return;
            }

            if (cbVersion.Text == "" || cbVersion.SelectedIndex == -1)
            {
                return;
            }

            
            GameProfile _gp = new GameProfile();

            if (checkAcount.Checked)
            {
                _gp.AcountIndex = cbAcounts.SelectedIndex;
            }
            else
            {
                _gp.AcountIndex = -1;
            }
            if (checkServer.Checked)
            {
                if (!tbServer.Text.Contains(":"))
                {
                    tbServer.Text += ":25565";
                }
                _gp.ServerAddress = tbServer.Text.Split(':')[0];
                _gp.ServerPort = Int32.Parse(tbServer.Text.Split(':')[1]);
            }
            _gp.JoinServer = checkServer.Checked;
            _gp.Width = (int)nuWidth.Value;
            _gp.Height = (int)nuHeight.Value;
            _gp.JavaXms = (int)nuXms.Value;
            _gp.JavaXmx = (int)nuXmx.Value;
            _gp.Version = cbVersion.Text;
            _gp.DisplayName = tbName.Text;

            _gp.ShowDOS = gp.ShowDOS;
            _gp.JavaMaxNewSize = gp.JavaMaxNewSize;
            _gp.JavaMaxPermSize = gp.JavaMaxPermSize;
            _gp.JavaNewRatio = gp.JavaNewRatio;
            _gp.JavaNewSize = gp.JavaNewSize;
            _gp.JavaPermSize = gp.JavaPermSize;
            _gp.JavaServivorRatio = gp.JavaServivorRatio;
            result = _gp;

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Hide();
        }

        public GameProfile GetResultProfile()
        {
            return result;
        }

        private void btnClean_Click(object sender, EventArgs e)
        {
            if (MessageBox.Show("データを修復しますか？\r\n現在のデータはすべて削除されます？", "警告", MessageBoxButtons.YesNo, MessageBoxIcon.Warning, MessageBoxDefaultButton.Button2) == System.Windows.Forms.DialogResult.Yes)
            {
                string[] dirs = System.IO.Directory.GetDirectories(directory + "/");
                for (int i = 0; i < dirs.Length; i++)
                {
                    if (!dirs[i].EndsWith("mods") && !dirs[i].EndsWith("resourcepacks") && !dirs[i].EndsWith("saves"))
                    {
                        System.IO.Directory.Delete(dirs[i], true);
                    }
                }
                string[] files = System.IO.Directory.GetFiles(directory + "/", "*.*", System.IO.SearchOption.TopDirectoryOnly);
                for (int i = 0; i < files.Length; i++)
                {
                    if (files[i].EndsWith(gp.Version + ".json"))
                    {
                        System.IO.File.Delete(files[i]);
                    }
                }
                gp.InForge = false;
                gp.ForgeVersion = "";

                string path = directory + "/data.silvermc";
                StreamWriter sw = new StreamWriter(path, false);
                sw.Write(JsonConvert.SerializeObject(gp, Formatting.Indented));
                sw.Close();
            }
        }
    }
}
