﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Net;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Converters;
using Ionic.Zip;

namespace SilverMC
{
    public class GameRunning
    {
        delegate void SetLabel(string name);
        delegate void ProgressClear(int max);
        delegate void WindowClose();
        delegate void Dialog(string message);
        delegate void ProgressStyle(ProgressBarStyle style);

        public const string FORGE_LIB_URL = "http://files.minecraftforge.net/maven/";
        public const string FORGE_LIB_URL2 = "http://repo.maven.apache.org/maven2/";

        private string gameDirectory;
        private GameData game;
        private GameProfile profile;
        private Acount acount;
        private SilverMCData configData;
        private AuthResponce acountSession;

        private Label strLabel;
        private ProgressBar ber;
        private Form baseForm;
        private bool exception;

        public GameRunning(string directory, GameProfile version, Acount loginAcount, SilverMCData config)
        {
            this.gameDirectory = directory;
            this.profile = version;
            this.acount = loginAcount;
            this.configData = config;
        }

        public void SetControl(Form form, Label label, ProgressBar pb)
        {
            this.baseForm = form;
            this.strLabel = label;
            this.ber = pb;
            SetProgressBerStyle(ProgressBarStyle.Continuous);
        }

        public bool Run()
        {
            if (!Login())
            {
                baseForm.Invoke(new Dialog(MesShow), "ログインに失敗しました");
                return false;
            }
            if (!DownloadVersion())
            {
                baseForm.Invoke(new Dialog(MesShow), "バージョン情報の確認に失敗しました");
                return false;
            }
            DowloadResource();
            DownloadLibraries();
            if (exception)
            {
                baseForm.Invoke(new Dialog(MesShow), "ライブラリのダウンロードに失敗しました");
                return false;
            }

            string command = MakeCommand();
            Console.WriteLine(command);
            System.Diagnostics.Process p = new System.Diagnostics.Process();
            Log("起動しています");
            p.StartInfo.FileName = System.Environment.GetEnvironmentVariable("ComSpec");
            p.StartInfo.UseShellExecute = false;
            p.StartInfo.RedirectStandardOutput = true;
            p.StartInfo.RedirectStandardInput = false;
            p.StartInfo.WorkingDirectory = Path.GetFullPath(gameDirectory + "/");
            p.StartInfo.CreateNoWindow = !profile.ShowDOS;
            p.StartInfo.Arguments = @"/c " + command;
            p.Start();
            if (configData.StartWait)
            {
                string results = p.StandardOutput.ReadToEnd();
            }
            return true;
        }

        private bool Login()
        {
            ProgressReset(10);
            SetProgressBerStyle(ProgressBarStyle.Marquee);
            Log("ログインしています");
            if (configData.OfflinePlay)
            {
                acount.AccessToken = "123456789abcde";
                acount.DisplayName = "Player";
                acount.SessionId = "123456789";
                acount.UserType = "legacy";
                return true;
            }
            WebClient wc = new WebClient();
            string loginString = null;
            /*
            try
            {
                loginString = wc.DownloadString(String.Format(Constants.LOGIN_URL, acount.UserName, acount.PassWord));
            }
            catch (Exception) { }

            if (loginString == null || loginString == "")
            {
                return false;
            }

            if (loginString.Contains(":"))
            {
                string[] loginInfo = loginString.Split(':');
                if (loginInfo.Length == 5)
                {
                    Console.WriteLine(loginString);
                    acount.AccessToken = loginInfo[3];
                    acount.DisplayName = loginInfo[2];
                    acount.SessionId = loginInfo[4];
                    acount.UserType = "legacy";
                    return true;
                }
                else
                {
                    Console.WriteLine(loginString);
                }
            }
            elseif (loginString.Length != 0)*/
            if(true)
            {
                Session s = new Session();
                s.agent = new GameSession();
                s.agent.name = "Minecraft";
                s.agent.version = 1;
                s.password = acount.PassWord;
                s.username = acount.UserName;
                s.requestUser = true;
                s.clientToken = configData.ClientId.ToString();

                string file = JsonConvert.SerializeObject(s, Formatting.None);
                string jsonRequest = GetJsonRequest(file, Constants.AUTH_URL_AUTHENTICATE);
                Console.WriteLine(jsonRequest);
                if (jsonRequest == null)
                {
                    return false;
                }

                acountSession = JsonConvert.DeserializeObject<AuthResponce>(jsonRequest);
                if (acountSession == null || acountSession.AccessToken == null || acountSession.SelectedProfile == null)
                {
                    return false;
                }
                acount.SessionId = acountSession.SelectedProfile.Id;
                acount.AccessToken = acountSession.AccessToken;
                acount.UserName = acountSession.SelectedProfile.Id;
                acount.DisplayName = acountSession.SelectedProfile.Name;
                acount.UserType = acountSession.GetAcountType();

                if (acountSession.User != null && acountSession.User.Properties != null)
                {
                    acount.Properties = JsonConvert.SerializeObject(acountSession.User.Properties, Formatting.None);
                }

                Console.WriteLine(acountSession);

                if (acountSession.User != null)
                {
                    acount.UserName = acountSession.User.Id;
                }

                //RefrashRequest request = new RefrashRequest(acountSession.SelectedProfile) { ClinetToken = configData.ClientId.ToString(), AccessToken = acountSession.AccessToken };
                return true;
            }
            return false;
        }

        private bool DownloadVersion()
        {
            if (!File.Exists(gameDirectory + "/" + profile.Version + ".json"))
            {
                if (configData.OfflinePlay)
                {
                    return false;
                }
                else
                {
                    Log("バージョンデータを読み込んでいます");
                    ProgressReset(1);
                    using (WebClient wc = new WebClient())
                    {
                        wc.DownloadFile(String.Format(Constants.VERSION_FILE_JSON, profile.Version), gameDirectory + "/" + profile.Version + ".json");
                    }
                    ProgressAdd();
                }
            }
            Log("バージョンデータを整理しています");
            StreamReader sr = new StreamReader(gameDirectory + "/" + profile.Version + ".json");
            string json = sr.ReadToEnd();
            sr.Close();
            JObject jsonObj = JObject.Parse(json);
            game = new GameData();
            game.MainClass = jsonObj.GetValue("mainClass").ToString();
            game.Arguments = jsonObj.GetValue("minecraftArguments").ToString();
            game.Assets = jsonObj.GetValue("assets") != null ? jsonObj.GetValue("assets").ToString() : "legacy";
            JArray libs = jsonObj.GetValue("libraries") as JArray;
            game.Libraries = new List<Library>(libs.Count);
            ProgressReset(libs.Count);
            for (int i = 0; i < libs.Count; i++)
            {
                Library lib = new Library();
                JObject libData = libs[i] as JObject;
                lib.Name = libData.GetValue("name").ToString();
                lib.Url = libData.GetValue("url") != null ? (libData.GetValue("url").ToString() == FORGE_LIB_URL) ? FORGE_LIB_URL2 : (libData.GetValue("url").ToString()) : Constants.LIBRARY_FILE;

                JToken rules;
                if (libData.TryGetValue("rules", out rules))
                {
                    lib.Rules = new List<Rule>();
                    JArray rule2 = rules as JArray;
                    for (int j = 0; j < rule2.Count; j++)
                    {
                        Rule rule = new Rule();
                        rule.Action = (rule2[j] as JObject).GetValue("action") != null ? (rule2[j] as JObject).GetValue("action").ToString() : null;
                        if ((rule2[j] as JObject).GetValue("os") != null)
                        {
                            OS os = new OS();
                            os.Name = ((rule2[j] as JObject).GetValue("os") as JObject).GetValue("name").ToString();
                            os.Version = ((rule2[j] as JObject).GetValue("os") as JObject).GetValue("version") != null ? ((rule2[j] as JObject).GetValue("os") as JObject).GetValue("version").ToString() : null;
                            rule.Os = os;
                        }
                        lib.Rules.Add(rule);
                    }
                }
                JToken natives;
                if (libData.TryGetValue("natives", out natives))
                {
                    lib.Natives = new Native();
                    lib.Natives.Windows = (natives as JObject).GetValue("windows") != null ? (natives as JObject).GetValue("windows").ToString() : null;
                    lib.Natives.Linux = (natives as JObject).GetValue("linux") != null ? (natives as JObject).GetValue("linux").ToString() : null;
                    lib.Natives.OSX = (natives as JObject).GetValue("osx") != null ? (natives as JObject).GetValue("osx").ToString() : null;
                }
                JToken extract;
                if (libData.TryGetValue("extract", out extract))
                {
                    lib.Extract = new Extract();
                    JToken ex_ = (extract as JObject).GetValue("exclude");
                    if (ex_ != null)
                    {
                        lib.Extract.Exclude = new List<string>();
                        lib.Extract.Exclude.Add(ex_.ToString());
                    }
                }
                game.Libraries.Add(lib);
                ProgressAdd();
            }
            return true;
        }

        private void DowloadResource()
        {
            ProgressReset(10);
            SetProgressBerStyle(ProgressBarStyle.Marquee);
            Log("リソースを取得しています");
            WebClient wc = new WebClient();


            string rootPath = Constants.BASE_PATH + "System/assets/";
            if (!Directory.Exists(rootPath + "indexes"))
            {
                Directory.CreateDirectory(rootPath + "indexes");
            }
            if (!Directory.Exists(rootPath + "objects"))
            {
                Directory.CreateDirectory(rootPath + "objects");
            }
            if (!Directory.Exists(rootPath + "virtual"))
            {
                Directory.CreateDirectory(rootPath + "virtual");
            }
            wc.DownloadFile(String.Format(Constants.ASSETS_JSON, game.Assets), rootPath + "indexes/" + game.Assets + ".json");
            StreamReader sr = new StreamReader(rootPath + "indexes/" + game.Assets + ".json");
            string assetsJson = sr.ReadToEnd();
            sr.Close();
            JObject json = JObject.Parse(assetsJson);
            JObject resources = json.GetValue("objects") as JObject;
            ProgressReset(resources.Count);
            SetProgressBerStyle(ProgressBarStyle.Continuous);
            IList<string> keys = resources.Properties().Select(p => p.Name).ToList();
            Log("リソースをダウンロードしています (0/" + keys.Count + ")");
            for (int i = 0; i < keys.Count; i++)
            {
                string hash = (resources.GetValue(keys[i]) as JObject).GetValue("hash").ToString();
                string filePath = Path.GetFullPath(rootPath + "objects/" + hash.Substring(0, 2) + "/" + hash);
                string assetsFilePath = Path.GetFullPath(rootPath + "virtual/" + "legacy" + "/" + keys[i]);
                if (!File.Exists(filePath))
                {
                    if (!Directory.Exists(Path.GetDirectoryName(filePath)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(filePath));
                    }
                    try
                    {
                        wc.DownloadFile(String.Format(Constants.ASSETS_FILE, hash.Substring(0, 2), hash), filePath);
                    }
                    catch (Exception) { }
                }
                if (!File.Exists(assetsFilePath))
                {
                    if (!Directory.Exists(Path.GetDirectoryName(assetsFilePath)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(assetsFilePath));
                    }

                    try
                    {
                        File.Copy(filePath, assetsFilePath, true);
                    }
                    catch (Exception e)
                    {
                        Console.WriteLine(e.Message);
                    }
                }
                ProgressAdd();
                Log("リソースをダウンロードしています (" + (i + 1) + "/" + keys.Count + ")");
            }
            wc.Dispose();
        }

        private void DownloadLibraries()
        {
            WebClient wc = new WebClient();
            Log("フォルダを確認しています");
            string[] dirsPasses = { "bin", "bin/natives", "libraries", "mods" };
            ProgressReset(dirsPasses.Length);
            for (int i = 0; i < dirsPasses.Length; i++)
            {
                if (!Directory.Exists(gameDirectory + "/" + dirsPasses[i]))
                {
                    Directory.CreateDirectory(gameDirectory + "/" + dirsPasses[i]);
                }
                ProgressAdd();
            }
            Log("ライブラリをダウンロードしています (0/" + game.Libraries.Count + ")");
            SetProgressBerStyle(ProgressBarStyle.Continuous);
            string libsDir = gameDirectory + "/libraries/";
            string nativesDir = gameDirectory + "/bin/natives/";
            ProgressReset(game.Libraries.Count);
            for (int i = 0; i < game.Libraries.Count; i++)
            {
                string libName = game.Libraries[i].Name;
                string libBaseUrl = game.Libraries[i].Url;
                if (libName != null && libName.Contains(":") && libBaseUrl != null)
                {
                    if (game.Libraries[i].Rules != null)
                    {
                        bool exit = false;
                        for (int j = 0; j < game.Libraries[i].Rules.Count; j++)
                        {
                            if (game.Libraries[i].Rules[j].Action == "allow" && game.Libraries[i].Rules[j].Os != null)
                            {
                                if (game.Libraries[i].Rules[j].Os.Name == "osx" || game.Libraries[i].Rules[j].Os.Name == "linux")
                                {
                                    exit = true;
                                }
                            }
                            else if (game.Libraries[i].Rules[j].Action == "disallow" && game.Libraries[i].Rules[j].Os != null)
                            {
                                if (game.Libraries[i].Rules[j].Os.Name == "windows")
                                {
                                    exit = true;
                                }
                            }
                        }
                        if (exit)
                        {
                            ProgressAdd();
                            continue;
                        }
                    }
                    string[] lib = libName.Split(':');
                    string natives = "";
                    if (game.Libraries[i].Natives != null && game.Libraries[i].Natives.Windows != null)
                    {
                        natives = "-" + game.Libraries[i].Natives.Windows;
                        natives = natives.Replace("${arch}", System.Environment.Is64BitOperatingSystem ? "64" : "32");
                    }
                    string fileName = lib[1] + "-" + lib[2] + natives + ".jar";
                    string fileUrl = String.Join("/", lib[0].Split('.')) + "/" + lib[1] + "/" + lib[2] + "/" + fileName;
                    if (!Directory.Exists(Path.GetDirectoryName(libsDir + fileUrl)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(libsDir + fileUrl));
                    }
                    Console.WriteLine(libsDir + fileUrl);
                    if (!File.Exists(libsDir + fileUrl))
                    {
                        Console.WriteLine(game.Libraries[i].Url + fileUrl);
                        try
                        {
                            wc.DownloadFile(game.Libraries[i].Url + fileUrl, libsDir + fileUrl);
                        }
                        catch (Exception)
                        {
                            exception = true;
                        }
                    }
                    if (game.Libraries[i].Natives != null)
                    {
                        ZipFile zip = new ZipFile(libsDir + fileUrl);
                        zip.ExtractAll(nativesDir, ExtractExistingFileAction.DoNotOverwrite);
                        if (Directory.Exists(nativesDir + "/META-INF"))
                        {
                            Directory.Delete(nativesDir + "/META-INF", true);
                        }
                        if (game.Libraries[i].Extract != null && game.Libraries[i].Extract.Exclude != null)
                        {
                            for (int j = 0; j < game.Libraries[i].Extract.Exclude.Count; j++)
                            {
                                string removerDir = game.Libraries[i].Extract.Exclude[j];
                                removerDir = removerDir.TrimEnd(new char[] { '/', '\\' });
                                if (Directory.Exists(nativesDir + "/" + removerDir))
                                {
                                    Directory.Delete(nativesDir + "/" + removerDir, true);
                                }
                            }
                        }
                    }
                    Log("ライブラリをダウンロードしています (" + i + "/" + game.Libraries.Count + ")");
                    ProgressAdd();
                }
            }

            Log("Minecraft本体をダウンロードしています");
            ProgressReset(10);
            SetProgressBerStyle(ProgressBarStyle.Marquee);
            string jarPath = gameDirectory + "/bin/" + profile.Version + ".jar";
            if (!File.Exists(jarPath))
            {
                string jarUrl = String.Format(Constants.JAR_FILE, profile.Version);
                wc.DownloadFile(jarUrl, jarPath);
            }
            wc.Dispose();
        }

        private string GetJsonRequest(string post, string url)
        {
            byte[] postDataBytes = System.Text.Encoding.ASCII.GetBytes(post);

            WebRequest wr = WebRequest.Create(url);
            wr.Method = "POST";
            wr.ContentType = "application/json";
            wr.ContentLength = postDataBytes.Length;

            Stream reqStream = wr.GetRequestStream();
            reqStream.Write(postDataBytes, 0, postDataBytes.Length);
            reqStream.Close();
            try
            {
                WebResponse res = wr.GetResponse();
                Stream resStream = res.GetResponseStream();
                StreamReader sr = new System.IO.StreamReader(resStream);
                string result = sr.ReadToEnd();

                sr.Close();
                return result;
            }
            catch (Exception)
            {

            }
            return null;
        }

        private string MakeCommand()
        {
            Log("起動チェックをしています");
            string libDir = Path.GetFullPath(gameDirectory + "/libraries");
            string nativesDir = Path.GetFullPath(gameDirectory + "/bin/natives");
            string libsString = "";
            string[] libs = Directory.GetFiles(libDir, "*.jar", SearchOption.AllDirectories);
            for (int i = 0; i < libs.Length; i++)
            {
                if (libs[i] != null && libs[i] != "")
                {
                    libsString += libs[i] + ";";
                }
            }
            libsString += Path.GetFullPath(gameDirectory + "/bin/" + profile.Version + ".jar");
            string args = game.Arguments;
            StringBuilder sb = new StringBuilder(args);
            sb.Replace("${auth_player_name}", acount.DisplayName);
            sb.Replace("${auth_session}", String.Format("token:{0}:{1}", acount.AccessToken, acount.SessionId));
            sb.Replace("${version_name}", profile.Version);
            sb.Replace("${game_directory}", "\"" + Path.GetFullPath(gameDirectory) + "\"");
            sb.Replace("${game_assets}", "\"" + Path.GetFullPath(Constants.BASE_PATH + "System/assets/virtual/legacy") + "\"");
            sb.Replace("${assets_root}", "\"" + Path.GetFullPath(Constants.BASE_PATH + "System/assets") + "\"");
            sb.Replace("${assets_index_name}", "\"" + game.Assets + "\"");
            sb.Replace("${auth_uuid}", acount.SessionId);
            sb.Replace("${auth_access_token}", acount.AccessToken);
            sb.Replace("${user_properties}", acount.Properties == null ? "{}" : acount.Properties);
            sb.Replace("${user_type}", acount.UserType);

            string addLine = String.Format(" --width {0} --height {1} ", profile.Width, profile.Height);
            if (profile.JoinServer)
            {
                addLine += String.Format("--server \"{0}\" --port {1} ", profile.ServerAddress, profile.ServerPort);
            }
            string optionLine = String.Format("-Xmx{0}M -Xms{1}M -Xmn{2}M -XX:MaxNewSize={3}m -XX:NewRatio={4} -XX:SurvivorRatio={5} -XX:PermSize={6}m -XX:MaxPermSize={7}m -XX:HeapDumpPath=MojangTricksIntelDriversForPerformance_javaw.exe_minecraft.exe.heapdump", profile.JavaXmx, profile.JavaXms, profile.JavaNewSize, profile.JavaMaxNewSize, profile.JavaNewRatio, profile.JavaServivorRatio, profile.JavaPermSize, profile.JavaPermSize);

            string commandLine = String.Format("\"\"" + configData.JavaPath + "\" " + optionLine + " -Djava.library.path=\"{0}\" -cp \"{1}\" {2} {3}" + addLine + "\"", nativesDir, libsString, game.MainClass, sb.ToString());

            return commandLine;
        }

        private void Log(string name)
        {
            baseForm.Invoke(new SetLabel(SetLog), name);
        }

        private void SetLog(string name)
        {
            strLabel.Text = name;
        }

        private void ProgressAdd()
        {
            baseForm.Invoke((MethodInvoker)delegate()
            {
                ber.Value++;
            });
        }

        private void SetProgressBerStyle(ProgressBarStyle par1)
        {
            baseForm.Invoke(new ProgressStyle(ProgressSetting), par1);
        }

        private void ProgressSetting(ProgressBarStyle berStyle)
        {
            ber.Style = berStyle;
        }

        private void ProgressReset(int max)
        {
            baseForm.Invoke(new ProgressClear(ProgressSet), max);
        }

        private void ProgressSet(int max)
        {
            ber.Maximum = max;
            ber.Value = 0;
        }

        private void MesShow(string name)
        {
            MessageBox.Show(name, "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
        }
    }
}
