﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Converters;
using System.Net;

namespace SilverMC
{
    public partial class MainWindow : Form
    {
        delegate void DelegateVersion(string json);

        private Versions versionData;
        private List<string> displayPair;
        private SilverMCData config;
        private GameProfile[] profiles;
        private ListData[] drawProfiles;
        private Button[] btns;
        private bool resizeNow = false;
        public const string VERSION = "1.2";
        public const int BUILD_ID = 3;

        public string BASE_PATH = System.Windows.Forms.Application.StartupPath + "\\";

        public MainWindow()
        {
            InitializeComponent();
            this.SetStyle(ControlStyles.OptimizedDoubleBuffer, true);
            this.SetStyle(ControlStyles.AllPaintingInWmPaint, true);
            this.SetStyle(ControlStyles.UserPaint, true);
            this.ResizeBegin += new EventHandler(MainWindow_ResizeBegin);
            this.ResizeEnd += new EventHandler(MainWindow_ResizeEnd);
        }

        private void MainWindow_Load(object sender, EventArgs e)
        {

            if (!Directory.Exists(BASE_PATH + "Games"))
            {
                Directory.CreateDirectory(BASE_PATH + "Games");
            }
            if (!Directory.Exists(BASE_PATH + "System/assets"))
            {
                Directory.CreateDirectory(BASE_PATH + "System/assets");
            }
            if (!File.Exists(BASE_PATH + "System/config.dat"))
            {
                config = new SilverMCData();
                config.Acounts = new List<Acount>();
                config.Buttons = new string[] { "btnRun", "btnEdit", "btnNew", "btnAcount", "btnMod", "btnFolder", "btnRefresh", "btnOption"};
                config.ClientId = Guid.NewGuid();
                config.DefaultAcount = -1;
                config.JavaPath = GetJavaPath();
                config.ListDrawMode = 1;

                using (StreamWriter sw = new StreamWriter(BASE_PATH + "System/config.dat"))
                {
                    sw.Write(JsonConvert.SerializeObject(config, Formatting.Indented));
                    sw.Close();
                }
            }
            else
            {
                using (StreamReader sr = new StreamReader(BASE_PATH + "System/config.dat"))
                {
                    config = JsonConvert.DeserializeObject<SilverMCData>(sr.ReadToEnd());
                    sr.Close();
                }
            }
            if (config.JavaPath == null)
            {
                config.JavaPath = GetJavaPath();
            }
            if (config.DrawString == null)
            {
                config.DrawString = "{version} / {dirname} / {acount} / {server}";
                //config.DrawString = "";
            }
            if (config.QuickPlays == null)
            {
                config.QuickPlays = new List<QuickPlayData>();
            }

            if (config.DefaultAcount != -1 && System.Environment.GetCommandLineArgs().Length > 1 && System.Environment.CommandLine.Contains("mccqp:"))
            {
                string[] args = System.Environment.GetCommandLineArgs();
                string[] array = args[1].Split('/');

                if (array[0] == "mccqp:")
                {
                    QuickPlayData qp = new QuickPlayData(array);
                    QuickPlayWindow qw = new QuickPlayWindow(qp, config);
                    if (qw.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {

                    }
                    Application.Exit();
                }
            }

            backgroundInit.RunWorkerAsync();

            ButtonMove();
            ListReload();
        }

        private void ButtonMove()
        {
            if (btns == null)
            {
                btns = new Button[8];
                for (int i = 0; i < btns.Length; i++)
                {
                    btns[i] = Controls[config.Buttons[i]] as Button;
                }
            }
            for (int i = 0; i < config.Buttons.Length; i++)
            {
                int x = i / 4 * 80 + this.Width - 179;
                int y = i % 4 * 76 + 12;
                btns[i].Location = new Point(x, y);
            }
        }

        private void VersionsConvert(string json)
        {
            versionData = JsonToVersions(json);
        }

        public static Versions JsonToVersions(string json)
        {
            JObject obj = JObject.Parse(json);
            JArray versions = obj.GetValue("versions") as JArray;
            Versions games = new Versions();
            games.Games = new List<Version>();
            for (int i = 0; i < versions.Count; i++)
            {
                Version nV = new Version();
                nV.Id = (versions[i] as JObject).GetValue("id").ToString();
                nV.Time = (versions[i] as JObject).GetValue("time").ToString();
                nV.ReleaseTime = (versions[i] as JObject).GetValue("releaseTime").ToString();
                nV.Type = (versions[i] as JObject).GetValue("type").ToString();
                games.Games.Add(nV);
            }
            return games;
        }

        private void backgroundInit_DoWork(object sender, DoWorkEventArgs e)
        {
            WebClient wc = new WebClient();
            string versionJson = wc.DownloadString(Constants.VERSION_JSON);
            try
            {
                this.Invoke(new DelegateVersion(VersionsConvert), versionJson);
            }
            catch (Exception) { }
        }

        private void btnNew_Click(object sender, EventArgs e)
        {
            if (versionData == null) return;
            NewGameWindow f = new NewGameWindow(versionData);
            if (f.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                string dirPass = BASE_PATH + "Games/" + f.GetGameName();
                Directory.CreateDirectory(dirPass);
                if (f.IsHandMode())
                {
                    GameProfile profiel = new GameProfile();
                    profiel.DisplayName = f.GetGameName();
                    profiel.Version = versionData.Games[f.GetGameVersionIndex()].Id;
                    profiel.Width = 854;
                    profiel.Height = 480;
                    profiel.JavaXms = 512;
                    profiel.JavaXmx = 1024;
                    profiel.JavaNewSize = 128;
                    profiel.JavaMaxNewSize = 128;
                    profiel.JavaPermSize = 64;
                    profiel.JavaMaxPermSize = 64;
                    profiel.JavaNewRatio = 8;
                    profiel.JavaServivorRatio = 8;
                    profiel.JoinServer = false;
                    profiel.AcountIndex = -1;
                    profiel.InForge = false;
                    profiel.ForgeVersion = "";
                    using (StreamWriter sw = new StreamWriter(dirPass + "/data.silvermc"))
                    {
                        sw.Write(JsonConvert.SerializeObject(profiel, Formatting.Indented));
                        sw.Close();
                    }
                }
                else
                {
                    using (StreamReader sr = new StreamReader(f.GetFileName()))
                    {
                        string original = sr.ReadToEnd();
                        sr.Close();
                        GameProfile profile = JsonConvert.DeserializeObject<GameProfile>(original);
                        profile.AcountIndex = -1;
                        StreamWriter sw = new StreamWriter(dirPass + "/data.silvermc");
                        sw.Write(JsonConvert.SerializeObject(profile, Formatting.Indented));
                        sw.Close();
                    }
                }
                ListReload();
            }
        }

        private void ListReload()
        {
            if (config.Acounts.Count < 0)
            {
                MessageBox.Show("アカウントが登録されていません\r\nアカウントボタンからアカウントを登録してください", "Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            if (config.DefaultAcount == -1 || !(config.DefaultAcount < config.Acounts.Count))
            {
                MessageBox.Show("デフォルトアカウントが指定されていません\r\nアカウントボタンから指定してください", "Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
                return;
            }
            listGames.Items.Clear();
            displayPair = new List<string>();
            string[] dirs = Directory.GetDirectories(BASE_PATH + "Games/");
            profiles = null;
            profiles = new GameProfile[dirs.Length];
            drawProfiles = new ListData[dirs.Length];
            listGames.BeginUpdate();
            for (int i = 0; i < dirs.Length; i++)
            {
                string path = dirs[i] + "/data.silvermc";
                if (!File.Exists(path))
                {
                    continue;
                }
                using (StreamReader sr = new StreamReader(path))
                {
                    string json = sr.ReadToEnd();
                    sr.Close();

                    GameProfile gp = JsonConvert.DeserializeObject<GameProfile>(json);
                    if (gp.ForgeVersion == null) gp.ForgeVersion = "";
                    profiles[i] = gp;
                    ListData listData = new ListData();
                    if (!(gp.AcountIndex < config.Acounts.Count))
                    {
                        gp.AcountIndex = -1;
                    }
                    listData.AcountName = Utils.DecryptString(gp.AcountIndex != -1 ? config.Acounts[gp.AcountIndex].UserName : config.Acounts[config.DefaultAcount].UserName, Utils.SOURCE_STRING);
                    listData.DirName = Path.GetFileName(dirs[i]);
                    listData.Server = gp.JoinServer ? gp.ServerAddress : "なし";
                    listData.Version = gp.Version;
                    drawProfiles[i] = listData;
                    if (config.ListDrawMode == 0)
                    {
                        listGames.Items.Add(Path.GetFileName(dirs[i]));
                    }
                    else if (config.ListDrawMode == 1)
                    {
                        listGames.Items.Add(gp.DisplayName);
                    }
                    else
                    {
                        listGames.Items.Add(gp.DisplayName + " (" + Path.GetFileName(dirs[i]) + ")");
                    }
                    displayPair.Add(dirs[i]);
                }
            }
            listGames.EndUpdate();
        }

        private void btnRun_Click(object sender, EventArgs e)
        {
            if (listGames.SelectedIndex != -1)
            {
                GameProfile gp = GetCurrentProfile();
                if (config.Acounts == null || config.Acounts.Count <= 0)
                {
                    MessageBox.Show("アカウントが登録されていません\r\nアカウントボタンからアカウントを登録してください", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                if (!(config.DefaultAcount < config.Acounts.Count))
                {
                    config.DefaultAcount = -1;
                    MessageBox.Show("指定されたアカウントが存在しません\r\nアカウントボタンから確認してください");
                    return;
                }
                Acount ac = null;
                if (gp.AcountIndex < config.Acounts.Count && gp.AcountIndex != -1)
                {
                    ac = config.Acounts[gp.AcountIndex];
                }
                else
                {
                    ac = config.Acounts[config.DefaultAcount];
                    gp.AcountIndex = -1;
                }
                Acount nAc = new Acount();
                nAc.UserName = Utils.DecryptString(ac.UserName, Utils.SOURCE_STRING);
                nAc.PassWord = Utils.DecryptString(ac.PassWord, Utils.SOURCE_STRING);
                RunWindow rw = new RunWindow(displayPair[listGames.SelectedIndex], gp, nAc, config);
                if (rw.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {

                }
                rw.Close();
                rw.Dispose();
                Utils.ForceGarbageCollection();
            }
        }

        private void MainWindow_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveConfig();
        }

        private void SaveConfig()
        {
            using (StreamWriter sw = new StreamWriter(BASE_PATH + "System/config.dat"))
            {
                sw.Write(JsonConvert.SerializeObject(config, Formatting.Indented));
                sw.Close();
            }
        }

        private void btnAcount_Click(object sender, EventArgs e)
        {
            AcountWindow f = new AcountWindow(config);
            if (f.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                SaveConfig();
            }
            f.Close();
            f.Dispose();
            Utils.ForceGarbageCollection();
        }

        private void btnEdit_Click(object sender, EventArgs e)
        {
            if (listGames.SelectedIndex != -1 && versionData != null)
            {
                EditWindow f = new EditWindow(config, GetCurrentProfile(), versionData, displayPair[listGames.SelectedIndex]);
                if (f.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    string path = displayPair[listGames.SelectedIndex] + "/data.silvermc";
                    using (StreamWriter sw = new StreamWriter(path, false))
                    {
                        sw.Write(JsonConvert.SerializeObject(f.GetResultProfile(), Formatting.Indented));
                        sw.Close();
                    }
                    ListReload();
                }
                f.Dispose();
                Utils.ForceGarbageCollection();
                
            }
        }

        private GameProfile GetCurrentProfile()
        {
            return GetCurrentProfile(listGames.SelectedIndex);
        }

        private GameProfile GetCurrentProfile(int index)
        {
            if (index == -1) return null;
            using (StreamReader sr = new StreamReader(displayPair[listGames.SelectedIndex] + "/data.silvermc"))
            {
                string json = sr.ReadToEnd();
                sr.Close();
                GameProfile gp = JsonConvert.DeserializeObject<GameProfile>(json);
                return gp;
            }
        }

        private string GetJavaPath()
        {
            string[] javas = { "C:\\Program Files\\Java\\jre7\\bin\\javaw.exe", "C:\\Program Files\\Java\\jre6\\bin\\javaw.exe", "C:\\Program Files (x86)\\Java\\jre7\\bin\\javaw.exe", "C:\\Program Files (x86)\\Java\\jre6\\bin\\javaw.exe" };
            for (int i = 0; i < javas.Length; i++)
            {
                if (File.Exists(javas[i]))
                {
                    return javas[i];
                }
            }
            MessageBox.Show("Javaが見つけられませんでした\r\nJavaをインストールしてください", "Error", MessageBoxButtons.OK, MessageBoxIcon.Error);
            Application.Exit();
            return "";
        }

        private void btnFolder_Click(object sender, EventArgs e)
        {
            if (listGames.SelectedIndex != -1)
            {
                Console.WriteLine(displayPair[listGames.SelectedIndex]);
                System.Diagnostics.Process.Start(Path.GetFullPath(displayPair[listGames.SelectedIndex] + "/"));
            }
        }

        private void btnRefresh_Click(object sender, EventArgs e)
        {
            ListReload();
        }

        private void btnMod_Click(object sender, EventArgs e)
        {
            if (listGames.SelectedIndex != -1)
            {
                ModWindow f = new ModWindow(GetCurrentProfile(), displayPair[listGames.SelectedIndex]);
                if (f.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {

                }
                f.Close();
                f.Dispose();
            }
        }

        private void listGames_DrawItem(object sender, DrawItemEventArgs e)
        {
            if (resizeNow) return;
            e.DrawBackground();

            if ((e.State & DrawItemState.Selected) == DrawItemState.Selected)
            {
                e.Graphics.FillRectangle(Brushes.LightPink, e.Bounds);
            }
            StringFormat sf = new StringFormat();
            sf.Trimming = StringTrimming.EllipsisCharacter;
            if (e.Index != -1)
            {
                Brush b = Brushes.Black;
                string txt = ((ListBox)sender).Items[e.Index].ToString();
                e.Graphics.DrawString(txt, new Font("MS UI Gothic", 12), b, new Rectangle(e.Bounds.X, e.Bounds.Y + 2, e.Bounds.Width, e.Bounds.Height));
                SizeF size = e.Graphics.MeasureString(txt, new Font("MS UI Gothic", 12));
                listGames.HorizontalExtent = Math.Max((int)size.Width + 10, listGames.HorizontalExtent);
                if (config.DrawString != "")
                {
                    string _txt = config.DrawString.Replace("{acount}", drawProfiles[e.Index].AcountName).Replace("{version}", drawProfiles[e.Index].Version).Replace("{server}", drawProfiles[e.Index].Server).Replace("{dirname}", drawProfiles[e.Index].DirName);

                    e.Graphics.DrawString(_txt, new Font("MS UI Gothic", 9), Brushes.Gray, new Rectangle(e.Bounds.X + 12, e.Bounds.Y + 19, e.Bounds.Width, e.Bounds.Height - 20), sf);
                }
            }
            sf.Dispose();
            //e.DrawFocusRectangle();
        }


        private void MainWindow_ResizeEnd(object sender, EventArgs e)
        {
            resizeNow = false;
            listGames.Invalidate();
            System.GC.Collect();
        }

        private void MainWindow_ResizeBegin(object sender, EventArgs e)
        {
            resizeNow = true;
        }

        private void MainWindow_SizeChanged(object sender, EventArgs e)
        {
            ButtonMove();
            listGames.Width = this.Width - 197;
            listGames.Height = this.Height - 66;
            
        }

        private void listGames_DoubleClick(object sender, EventArgs e)
        {
            btnRun.PerformClick();
        }

        private void btnOption_Click(object sender, EventArgs e)
        {
            OptionWindow ow = new OptionWindow(config);
            if (ow.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {

            }
            ow.Close();
            ow.Dispose();
        }

        private void strip_TargetRun_Click(object sender, EventArgs e)
        {
            btnRun.PerformClick();
        }

        private void strip_TargetEdit_Click(object sender, EventArgs e)
        {
            btnEdit.PerformClick();
        }

        private void strip_TargetCopy_Click(object sender, EventArgs e)
        {
            if (listGames.SelectedIndex != -1)
            {
                string src = displayPair[listGames.SelectedIndex];
                for (; ; )
                {
                    src += "-Copy";
                    if (!Directory.Exists(src))
                    {
                        break;
                    }
                }
                Utils.CopyDirectory(displayPair[listGames.SelectedIndex] + "/", src + "/");
                ListReload();
            }
        }

        private void listGames_MouseClick(object sender, MouseEventArgs e)
        {

        }

        private void listGames_MouseDown(object sender, MouseEventArgs e)
        {
            if ((e.Button & System.Windows.Forms.MouseButtons.Right) == System.Windows.Forms.MouseButtons.Right)
            {
                int index = listGames.IndexFromPoint(new Point(e.X, e.Y));
                if (index != -1)
                {
                    listGames.SelectedIndex = index;
                }
            }
        }

        private void strip_Reload_Click(object sender, EventArgs e)
        {
            btnRefresh.PerformClick();
        }

        private void strip_TagrtOpen_Click(object sender, EventArgs e)
        {
            btnFolder.PerformClick();
        }

        private void stripTargetEdit_Click(object sender, EventArgs e)
        {
            btnMod.PerformClick();
        }

        private void strip_TargetDel_Click(object sender, EventArgs e)
        {
            if(listGames.SelectedIndex != -1)
            {
                GameProfile p = GetCurrentProfile();
                if (MessageBox.Show(p.DisplayName + "を削除しますか？\r\nコンピューター上から削除されます", "警告", MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) == System.Windows.Forms.DialogResult.Yes)
                {
                    try
                    {
                        Directory.Delete(Path.GetFullPath(displayPair[listGames.SelectedIndex] + "/"), true);
                    }
                    catch (Exception) { }
                    ListReload();
                }
            }
        }
    }
}
