﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using Ionic.Zip;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Linq;
using SilverNBTLibrary;

namespace SilverMC
{
    public partial class ModWindow : Form
    {
        private GameProfile game;
        private string dir;

        private List<ModData> mods = new List<ModData>();
        private List<ServerInfo> servers = new List<ServerInfo>();

        public ModWindow(GameProfile gp, string _dir)
        {
            InitializeComponent();
            dir = _dir;
            game = gp;
            ReloadModList();
            ServerListLoad();
        }

        private void ServerListLoad()
        {
            servers.Clear();
            if (File.Exists(dir + "/servers.dat"))
            {
                using (FileStream fs = new FileStream(dir + "/servers.dat", FileMode.Open))
                {
                    try
                    {
                        NBTTagCompound tags = NBTFile.LoadToStreamForZip(fs);
                        if (tags.HasKey("servers"))
                        {
                            NBTTagList collection = tags.GetTag("servers") as NBTTagList;
                            if (collection != null)
                            {
                                for (int i = 0; i < collection.Count; i++)
                                {
                                    NBTTagCompound compound = collection.Get(i) as NBTTagCompound;
                                    if (compound != null)
                                    {
                                        ServerInfo si = new ServerInfo();
                                        if (compound.HasKey("name")) si.Name = compound.GetString("name");
                                        if (compound.HasKey("icon")) si.ServerIcon = compound.GetString("icon");
                                        if (compound.HasKey("ip")) si.IP = compound.GetString("ip");
                                        if (compound.HasKey("hideAddress")) si.HideAddress = compound.GetByte("hideAddress") == 1;
                                        if (compound.HasKey("acceptTextures")) si.AcceptTextures = compound.GetByte("acceptTextures") == 1;
                                        servers.Add(si);
                                    }
                                }
                            }
                        }
                    }
                    catch (Exception) { }
                    fs.Close();
                }
                ServerReload();
            }
        }

        private void ServerReload()
        {
            serverList.Items.Clear();
            for (int i = 0; i < servers.Count; i++)
            {
                ServerJoin sj = new ServerJoin(servers[i].IP);
                sj.Start();
                serverList.Items.Add(servers[i].Name);
            }
        }

        private void FinishModList()
        {
            for (int i = 0; i < modListView.Items.Count; i++)
            {
                mods[i].Enabled = modListView.Items[i].Checked;
            }

            for (int i = 0; i < mods.Count; i++)
            {
                string path = mods[i].Path;
                Console.WriteLine(path + File.Exists(path));
                if (!File.Exists(path))
                {
                    path += ".disabled";
                }

                if (!File.Exists(path))
                {
                    continue;
                }

                if (mods[i].Enabled)
                {
                    if (mods[i].Path.EndsWith(".disabled"))
                    {
                        File.Move(path, mods[i].Path.Replace(".disabled", ""));
                    }
                }
                else
                {
                    if (!mods[i].Path.EndsWith(".disabled"))
                    {
                        File.Move(path, mods[i].Path + ".disabled");
                    }
                }
            }

            this.DialogResult = System.Windows.Forms.DialogResult.OK;
            this.Hide();
        }

        private void btnFinish_Click(object sender, EventArgs e)
        {
            FinishModList();
        }

        private void btnForge_Click(object sender, EventArgs e)
        {
            tabControl1.SelectedIndex = 1;
            BtnChecked();
        }

        private void btnMod_Click(object sender, EventArgs e)
        {
            tabControl1.SelectedIndex = 0;
            BtnChecked();
        }

        private void BtnChecked()
        {
            btnForge.Enabled = tabControl1.SelectedIndex != 1;
            btnMod.Enabled = tabControl1.SelectedIndex != 0;
            btnServer.Enabled = tabControl1.SelectedIndex != 2;
        }

        private void btnRef_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                tbForgePass.Text = openFileDialog1.FileName;
                openFileDialog1.FileName = "";
            }
        }

        public void InstallForge(string forgePath)
        {
            if (File.Exists(forgePath))
            {
                string jarPath = Path.GetFullPath(dir + "/bin/" + game.Version + ".jar");
                Console.WriteLine(jarPath);
                if (!File.Exists(jarPath))
                {
                    MessageBox.Show(jarPath + "\r\nが見つかりません\r\n一度ゲームを起動してから再度試行してください");
                    return;
                }
                if (game.InForge)
                {
                    if (MessageBox.Show("すでにこのデータにForgeがインストールされている可能性があります\r\nインストールを続けますか？", "Error", MessageBoxButtons.YesNo, MessageBoxIcon.Information, MessageBoxDefaultButton.Button2) == System.Windows.Forms.DialogResult.No)
                    {
                        return;
                    }
                }
                ZipFile forge = new ZipFile(forgePath);
                ZipEntry jsonFile = forge["install_profile.json"];
                if (jsonFile != null)
                {
                    string jsonProfile = Extract(jsonFile);
                    JObject joj = JObject.Parse(jsonProfile);
                    InstallStruct forgeProfile = JsonConvert.DeserializeObject<InstallStruct>(joj.GetValue("install").ToString());
                    if (forgeProfile.Minecraft != game.Version)
                    {
                        MessageBox.Show("ForgeのバージョンとMinecraftのバージョンが一致しません");
                        return;
                    }
                    string extBaseFile = dir + "/libraries/net/minecraftforge/minecraftforge/" + forgeProfile.Version.Replace("Forge ", "") + "/minecraftforge-" + forgeProfile.Version.Replace("Forge ", "") + ".jar";
                    if (int.Parse(forgeProfile.Minecraft.Replace(".", "")) >= 172)
                    {
                        extBaseFile = dir + "/libraries/net/minecraftforge/forge/" + forgeProfile.Minecraft + "-" + forgeProfile.Version.Replace("Forge ", "") + "/forge-" + forgeProfile.Minecraft + "-" + forgeProfile.Version.Replace("Forge ", "") + ".jar";
                    }
                    if (!Directory.Exists(Path.GetDirectoryName(extBaseFile)))
                    {
                        Directory.CreateDirectory(Path.GetDirectoryName(extBaseFile));
                    }

                    ZipEntry forgeJar = forge[forgeProfile.FilePath];


                    if (forgeJar != null)
                    {
                        using (FileStream fs = new FileStream(extBaseFile, FileMode.Create))
                        {
                            forgeJar.Extract(fs);
                            fs.Close();
                        }

                        using (MemoryStream ms = new MemoryStream())
                        {

                            forgeJar.Extract(ms);
                            ms.Position = 0;
                            ZipFile f = ZipFile.Read(ms);

                            ZipEntry versionJson = f["version.json"];
                            if (File.Exists(Path.GetFullPath(dir + "/version.json")))
                            {
                                File.Delete(Path.GetFullPath(dir + "/version.json"));
                            }
                            versionJson.Extract(Path.GetFullPath(dir + "/"));
                            ms.Close();
                        }
                        File.Delete(Path.GetFullPath(dir + "/" + game.Version + ".json"));
                        File.Move(Path.GetFullPath(dir + "/version.json"), Path.GetFullPath(dir + "/" + game.Version + ".json"));
                        game.InForge = true;
                        game.ForgeVersion = forgeProfile.Target;

                        string path = dir + "/data.silvermc";
                        using (StreamWriter sw = new StreamWriter(path, false))
                        {
                            sw.Write(JsonConvert.SerializeObject(game, Formatting.Indented));
                            sw.Close();
                        }
                        MessageBox.Show("Forgeのインストールが完了しました", "Silver MC", MessageBoxButtons.OK, MessageBoxIcon.Information);
                    }
                }
            }
        }

        private void btnForgeInstall_Click(object sender, EventArgs e)
        {
            InstallForge(tbForgePass.Text);
        }

        private string Extract(ZipEntry ze)
        {
            MemoryStream ms = new MemoryStream();
            ze.Extract(ms);
            ms.Position = 0;
            StreamReader sr = new StreamReader(ms, Encoding.UTF8);
            string result = sr.ReadToEnd();
            sr.Close();
            ms.Close();
            return result;
        }

        private byte[] ExtractByte(ZipEntry ze)
        {
            MemoryStream ms = new MemoryStream();
            ze.Extract(ms);
            byte[] buf = ms.ToArray();
            ms.Close();
            return buf;
        }

        private void btnAddMod_Click(object sender, EventArgs e)
        {
            if (openFileDialog2.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                AddMod(openFileDialog2.FileName);
                openFileDialog2.FileName = "";
            }
        }

        private void AddMod(string path)
        {
            string fullPath = Path.GetFullPath(dir + "/mods/" + Path.GetFileName(path));
            File.Copy(path, fullPath);
            ModData md = GetModData(fullPath);
            ListViewItem lvi = new ListViewItem(new string[] { md.Name, md.Version });
            lvi.Checked = md.Enabled;
            modListView.Items.Add(lvi);
            mods.Add(md);
        }

        private ModData GetModData(string path)
        {
            ZipFile zip = new ZipFile(path);
            if (zip.ContainsEntry("mcmod.info"))
            {
                ZipEntry info = zip["mcmod.info"];
                string json = Extract(info);
                try
                {
                    JArray _j = JArray.Parse(json);
                    if (_j != null && _j.Count > 0)
                    {
                        ModData mod = new ModData();
                        mod.Name = (_j[0] as JObject).GetValue("name").ToString();
                        mod.Version = (_j[0] as JObject).GetValue("version").ToString();
                        mod.Path = path;
                        mod.Enabled = !path.EndsWith(".disabled");
                        zip.Dispose();
                        return mod;
                    }
                }
                catch (Exception) { }
            }
            ModData md = new ModData();
            md.Name = Path.GetFileName(path);
            md.Version = "";
            md.Path = path;
            md.Enabled = !path.EndsWith(".disabled");
            zip.Dispose();
            return md;
        }

        private void ReloadModList()
        {
            mods.Clear();
            modListView.Items.Clear();

            List<string> passes = new List<string>();
            string[] wilds = { "*.zip", "*.jar", "*.disabled" };
            for (int i = 0; i < wilds.Length; i++)
            {
                string[] files = Directory.GetFiles(Path.GetFullPath(dir + "/mods/"), wilds[i]);
                passes.AddRange(files);
            }
            string[] dirs = passes.ToArray();

            for (int i = 0; i < dirs.Length; i++)
            {
                if (dirs[i].EndsWith(".zip") || dirs[i].EndsWith(".jar") || dirs[i].EndsWith(".disabled"))
                {
                    ModData md = GetModData(dirs[i]);
                    ListViewItem lvi = new ListViewItem(new string[] { md.Name, md.Version });
                    lvi.Checked = md.Enabled;
                    modListView.Items.Add(lvi);
                    mods.Add(md);
                }
            }
        }

        private void listMod_ItemCheck(object sender, ItemCheckEventArgs e)
        {

        }

        private void btnChangeResult_Click(object sender, EventArgs e)
        {
            FinishModList();
        }

        private void btnServer_Click(object sender, EventArgs e)
        {
            tabControl1.SelectedIndex = 2;
            BtnChecked();
        }

        private void serverList_DrawItem(object sender, DrawItemEventArgs e)
        {

        }

    }

    public class InstallStruct
    {
        public string MirrorList { get; set; }
        public string Target { get; set; }
        public string FilePath { get; set; }
        public string Logo { get; set; }
        public string Welcome { get; set; }
        public string Version { get; set; }
        public string Path { get; set; }
        public string ProfileName { get; set; }
        public string Minecraft { get; set; }
    }

    public class ModData
    {
        public string Name { get; set; }
        public string Version { get; set; }
        public string Path { get; set; }
        public bool Enabled { get; set; }
    }

}
