﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Threading;
using Newtonsoft.Json;
using Newtonsoft.Json.Linq;
using Newtonsoft.Json.Converters;
using System.Net;


namespace SilverMC
{
    public partial class QuickPlayWindow : Form
    {
        private QuickPlayData qpData;
        private SilverMCData config;
        private Versions vs;

        public QuickPlayWindow(QuickPlayData qpd, SilverMCData co)
        {
            InitializeComponent();
            qpData = qpd;
            config = co;

            WebClient wc = new WebClient();
            string versionJson = wc.DownloadString(Constants.VERSION_JSON);
            wc.Dispose();
            vs = MainWindow.JsonToVersions(versionJson);
        }

        private void QuickPlayWindow_Load(object sender, EventArgs e)
        {
            groupBox1.Text = qpData.ServerName + " へ接続します";
            string sa = qpData.GetServerString();
            for (int i = 0; i < config.QuickPlays.Count; i++)
            {
                if (config.QuickPlays[i].GetServerString() == sa)
                {
                    qpData = config.QuickPlays[i];
                }
            }

            if (qpData.GameDirectory == null)
            {
                label1.Text = "" + qpData.ServerName + "へ接続するためのデータを新しく登録してください";
                btnNew.Text = "新規ゲームデータの登録";
            }
            else
            {
                label1.Text = "保存された" + qpData.ServerName + "へ接続するためのデータで接続します";
                btnNew.Text = "起動";
            }
        }

        private void btnNew_Click(object sender, EventArgs e)
        {
            if (qpData.GameDirectory == null)
            {
                NewGameWindow f = new NewGameWindow(vs, true);
                if (f.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                {
                    string dirPass = Constants.BASE_PATH + "Games/" + f.GetGameName();
                    qpData.GameDirectory = dirPass;
                    Directory.CreateDirectory(dirPass);
                    if (f.IsHandMode())
                    {
                        GameProfile profiel = new GameProfile();
                        profiel.DisplayName = f.GetGameName();
                        profiel.Version = vs.Games[f.GetGameVersionIndex()].Id;
                        profiel.Width = 854;
                        profiel.Height = 480;
                        profiel.JavaXms = 512;
                        profiel.JavaXmx = 1024;
                        profiel.JavaNewSize = 128;
                        profiel.JavaMaxNewSize = 128;
                        profiel.JavaPermSize = 64;
                        profiel.JavaMaxPermSize = 64;
                        profiel.JavaNewRatio = 8;
                        profiel.JavaServivorRatio = 8;
                        profiel.AcountIndex = -1;
                        profiel.InForge = false;
                        profiel.ForgeVersion = "";
                        profiel.JoinServer = true;
                        profiel.ServerAddress = qpData.ServerAddress;
                        profiel.ServerPort = qpData.ServerPort;
                        StreamWriter sw = new StreamWriter(dirPass + "/data.silvermc");
                        sw.Write(JsonConvert.SerializeObject(profiel, Formatting.Indented));
                        sw.Close();
                        config.QuickPlays.Add(qpData);
                        SaveConfig();
                        label1.Text = "保存された" + qpData.ServerName + "へ接続するためのデータで接続します";
                        btnNew.Text = "起動";
                    }
                }
            }
            else
            {
                if(File.Exists(qpData.GameDirectory + "/data.silvermc"))
                {
                    StreamReader sr = new StreamReader(qpData.GameDirectory + "/data.silvermc");
                    string json = sr.ReadToEnd();
                    sr.Close();

                    GameProfile gp = JsonConvert.DeserializeObject<GameProfile>(json);
                    Acount ac = config.Acounts[config.DefaultAcount];
                    Acount nAc = new Acount();
                    nAc.UserName = Utils.DecryptString(ac.UserName, Utils.SOURCE_STRING);
                    nAc.PassWord = Utils.DecryptString(ac.PassWord, Utils.SOURCE_STRING);
                    QuickRunWindow rw = new QuickRunWindow(qpData.GameDirectory, gp, nAc, config, qpData);
                    this.Hide();
                    if (rw.ShowDialog() == System.Windows.Forms.DialogResult.OK)
                    {
                        
                    }
                }
            }
        }

        private void SaveConfig()
        {
            StreamWriter sw = new StreamWriter(Constants.BASE_PATH + "System/config.dat");
            sw.Write(JsonConvert.SerializeObject(config, Formatting.Indented));
            sw.Close();
        }
    }
}
