﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace SilverNBTLibrary
{
    public abstract class NBTBase
    {
        public static string[] TagNames = { "TAG_End", "TAG_Byte", "TAG_Short", "TAG_Int", "TAG_Long", "TAG_Float", "TAG_Double", "TAG_Byte_Array", "TAG_String", "TAG_List", "TAG_Compound", "TAG_Int_Array"};
        public static NBTTagType[] TagTypes = { NBTTagType.TAG_End, NBTTagType.TAG_Byte, NBTTagType.TAG_Short, NBTTagType.TAG_Int, NBTTagType.TAG_Long, NBTTagType.TAG_Float, NBTTagType.TAG_Double, NBTTagType.TAG_Byte_Array, NBTTagType.TAG_String, NBTTagType.TAG_List, NBTTagType.TAG_Int_Array};

        public NBTBase Owner { get; set; }
        public string Name { get; set; }
        public bool Expand { get; set; }

        public NBTBase(string name)
        {
            this.Name = name;
        }

        public abstract byte GetId();

        public abstract object GetResult();

        public abstract void WriteData(BinaryWriter bw);

        public abstract void LoadData(BinaryReader br);

        public abstract bool SetValue(object obj);

        public static void WriteNamedTag(NBTBase tag, BinaryWriter bw)
        {
            bw.WriteHelper(tag.GetId());
            if (tag.GetId() != 0)
            {
                bw.WriteHelper(tag.Name);
                tag.WriteData(bw);
            }
        }

        public NBTBase SetName(string name)
        {
            if (name == null)
            {
                Name = "";
            }
            else
            {
                Name = name;
            }
            return this;
        }

        public static NBTBase CreateNewTag(byte type, string name)
        {
            if (name == null) name = "";
            if (type ==  0) return new NBTTagEnd();
            if (type ==  1) return new NBTTagByte(name);
            if (type ==  2) return new NBTTagShort(name);
            if (type ==  3) return new NBTTagInt(name);
            if (type ==  4) return new NBTTagLong(name);
            if (type ==  5) return new NBTTagFloat(name);
            if (type ==  6) return new NBTTagDouble(name);
            if (type ==  7) return new NBTTagByteArray(name);
            if (type ==  8) return new NBTTagString(name);
            if (type ==  9) return new NBTTagList(name);
            if (type == 10) return new NBTTagCompound(name);
            if (type == 11) return new NBTTagIntArray(name);
            return null;
        }

        public static NBTBase ReadNamedTag(BinaryReader br)
        {
            byte type = br.ReadByteHelper();
            if (type == 0)
            {
                return new NBTTagEnd();
            }
            else
            {
                string n = br.ReadStringHelper();
                NBTBase nbt = CreateNewTag(type, n);
                nbt.LoadData(br);
                return nbt;
            }
        }

        public string GetTagName()
        {
            return TagNames[GetId()];
        }

        public NBTTagType GetTagType()
        {
            return TagTypes[GetId()];
        }

        public override string ToString()
        {
            return String.Format("[{0}] {1}:{2}", GetTagName(), Name, GetResult());
        }

        public abstract NBTBase Clone();
    }
}
