﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;
using System.IO.Compression;

namespace SilverNBTLibrary
{
    public class NBTFile
    {
        public static void SaveToStream(Stream path, NBTTagCompound tag)
        {
            using (BinaryWriter bw = new BinaryWriter(path))
            {
                NBTBase.WriteNamedTag(tag, bw);
                bw.Close();
            }
        }

        public static void SaveToFile(String path, NBTTagCompound tag, bool compress = true)
        {
            using (MemoryStream inFile = new MemoryStream())
            {
                using (FileStream outPut = new FileStream(path, FileMode.Create))
                {
                    SaveToStream(inFile, tag);

                    byte[] buf = inFile.ToArray();
                    if (compress)
                    {
                        using (GZipStream gStream = new GZipStream(outPut, CompressionMode.Compress))
                        {
                            gStream.Write(buf, 0, buf.Length);
                            gStream.Close();
                        }
                    }
                    else
                    {
                        outPut.Write(buf, 0, buf.Length);
                    }
                    inFile.Close();
                    outPut.Close();
                }
            }
        }

        public static NBTTagCompound LoadToStreamForZip(Stream path)
        {
            NBTTagCompound nbt = null;
            using (BinaryReader br = new BinaryReader(path))
            {
                bool gzip = false;
                if (br.ReadByte() == 0x1F)
                {
                    if (br.ReadByte() == 0x8b)
                    {
                        gzip = true;
                    }
                }
                br.BaseStream.Position = 0;
                if (!gzip)
                {

                    nbt = LoadToStream(br.BaseStream);
                    br.Close();
                    return nbt;
                }
                else
                {
                    using (GZipStream gStream = new GZipStream(br.BaseStream, CompressionMode.Decompress))
                    {
                        using (MemoryStream ms = new MemoryStream())
                        {
                            byte[] buf = new byte[1024];
                            int num;
                            while ((num = gStream.Read(buf, 0, buf.Length)) > 0)
                            {
                                ms.Write(buf, 0, num);
                            }
                            ms.Position = 0;
                            nbt = LoadToStream(ms);
                            ms.Close();
                        }
                        gStream.Close();
                    }
                    br.Close();
                }
            }

            return nbt;
        }

        public static NBTTagCompound LoadToStream(Stream path)
        {
            using (BinaryReader br = new BinaryReader(path))
            {
                NBTBase baseTag = NBTBase.ReadNamedTag(br);
                br.Close();
                if (baseTag is NBTTagCompound)
                {
                    return baseTag as NBTTagCompound;
                }
            }
            return null;
        }
    }
}
