﻿using System.IO;

namespace SilverNBTLibrary
{
    public class NBTTagByteArray : NBTBase
    {
        private byte[] data;

        public NBTTagByteArray(string name) : base(name) { }

        public NBTTagByteArray(string name, byte[] value)
            : base(name)
        {
            data = value;
        }

        public override byte GetId()
        {
            return (byte)7;
        }

        public override object GetResult()
        {
            return data;
        }

        public override void WriteData(BinaryWriter bw)
        {
            bw.WriteHelper(data.Length);
            bw.Write(data);
        }

        public override void LoadData(BinaryReader br)
        {
            int count = br.ReadInt32Helper();
            data = new byte[count];
            data = br.ReadBytes(count);
        }

        public override bool SetValue(object obj)
        {
            if (obj is byte[])
            {
                data = (byte[])obj;
                return true;
            }
            return false;
        }

        public int Length
        {
            get { return data.Length; }
        }

        public int Get(int index)
        {
            return data[index];
        }

        public bool Set(int index, object value)
        {
            byte b;
            if (byte.TryParse(value.ToString(), out b))
            {
                data[index] = b;
                return true;
            }
            return false;
        }

        public override NBTBase Clone()
        {
            NBTTagByte clone = new NBTTagByte(Name);
            clone.Expand = this.Expand;
            byte[] newData = new byte[data.Length];
            System.Array.Copy(data, newData, data.Length);
            clone.SetValue(newData);
            return clone;
        }
    }
}
