﻿using System.IO;

namespace SilverNBTLibrary
{
    public class NBTTagIntArray : NBTBase
    {
        private int[] data;

        public NBTTagIntArray(string name) : base(name) { }

        public NBTTagIntArray(string name, int[] value)
            : base(name)
        {
            data = value;
        }

        public override byte GetId()
        {
            return (byte)11;
        }

        public override object GetResult()
        {
            return data;
        }

        public override void WriteData(BinaryWriter bw)
        {
            bw.WriteHelper(data.Length);
            for (int i = 0; i < data.Length; i++)
            {
                bw.WriteHelper(data[i]);
            }
        }

        public override void LoadData(BinaryReader br)
        {
            int count = br.ReadInt32Helper();
            data = new int[count];
            for (int i = 0; i < count; i++)
            {
                data[i] = br.ReadInt32Helper();
            }
        }

        public override bool SetValue(object obj)
        {
            if (obj is int[])
            {
                data = (int[])obj;
                return true;
            }
            return false;
        }

        public int Length
        {
            get { return data.Length; }
        }

        public int Get(int index)
        {
            return data[index];
        }

        public bool Set(int index, object value)
        {
            int b;
            if (int.TryParse(value.ToString(), out b))
            {
                data[index] = b;
                return true;
            }
            return false;
        }

        public override NBTBase Clone()
        {
            NBTTagIntArray clone = new NBTTagIntArray(Name);
            clone.Expand = this.Expand;
            int[] newData = new int[data.Length];
            System.Array.Copy(data, newData, data.Length);
            clone.SetValue(newData);
            return clone;
        }
    }
}
