﻿using System.IO;
using System.Collections;
using System.Collections.Generic;
namespace SilverNBTLibrary
{
    public class NBTTagList : NBTBase, IEnumerable
    {
        private List<NBTBase> TagList = new List<NBTBase>();
        public byte TagType { get; set; }

        public NBTTagList() : base("") { }

        public NBTTagList(string name) : base(name) { }

        public override byte GetId()
        {
            return (byte)9;
        }

        public override object GetResult()
        {
            return TagList;
        }

        public override void WriteData(BinaryWriter bw)
        {
            if (TagList.Count > 0)
            {
                TagType = TagList[0].GetId();
            }
            else
            {
                TagType = (byte)1;
            }
            bw.WriteHelper(TagType);
            bw.WriteHelper(TagList.Count);
            foreach (NBTBase nbt in TagList)
            {
                nbt.WriteData(bw);
            }
        }

        public override void LoadData(BinaryReader br)
        {
            TagList.Clear();
            TagType = br.ReadByteHelper();
            int count = br.ReadInt32Helper();
            TagList = new List<NBTBase>();
            for (int i = 0; i < count; i++)
            {
                NBTBase nbt = NBTBase.CreateNewTag(TagType, null);
                nbt.Owner = this;
                nbt.LoadData(br);
                TagList.Add(nbt);
            }
        }

        public override bool SetValue(object obj)
        {
            if (obj is List<NBTBase>)
            {
                TagList = (List<NBTBase>)obj;
                return true;
            }
            return false;
        }

        public void Add(NBTBase tag)
        {
            tag.Owner = this;
            TagList.Add(tag);
        }

        public void Remove(NBTBase tag)
        {
            TagList.Remove(tag);
        }

        public void RemoveAt(int index)
        {
            TagList.RemoveAt(index);
        }

        public void Clear()
        {
            TagList.Clear();
        }

        public void Insert(int index, NBTBase tag)
        {
            TagList.Insert(index, tag);
        }

        public List<NBTBase> GetCollection()
        {
            return TagList;
        }

        public NBTBase Get(int index)
        {
            return TagList[index];
        }

        public bool SetAt(int index, object obj)
        {
            return TagList[index].SetValue(obj);
        }

        public IEnumerator GetEnumerator()
        {
            return TagList.GetEnumerator();
        }

        public int Count
        {
            get { return TagList.Count; }
        }

        public override NBTBase Clone()
        {
            NBTTagList newTag = new NBTTagList(Name);
            newTag.Expand = this.Expand;
            newTag.TagType = TagType;
            List<NBTBase> listsNew = new List<NBTBase>();
            for (int i = 0; i < TagList.Count; i++)
            {
                listsNew.Add(TagList[i].Clone());
            }
            newTag.SetValue(listsNew);
            return newTag;
        }
    }
}
