﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.IO;

namespace SilverNBTLibrary
{
    public static class StreamHelper
    {
        public static int ReadInt32Helper(this BinaryReader br)
        {
            return BitConverter.ToInt32(ReadBytes(br, 4), 0);
        }

        public static short ReadInt16Helper(this BinaryReader br)
        {
            return BitConverter.ToInt16(ReadBytes(br, 2), 0);
        }

        public static long ReadInt64Helper(this BinaryReader br)
        {
            return BitConverter.ToInt64(ReadBytes(br, 8), 0);
        }

        public static double ReadDoubleHelper(this BinaryReader br)
        {
            return BitConverter.ToDouble(ReadBytes(br, 8), 0);
        }

        public static float ReadSingleHelper(this BinaryReader br)
        {
            return BitConverter.ToSingle(ReadBytes(br, 4), 0);
        }

        public static float ReadCharHelper(this BinaryReader br)
        {
            return BitConverter.ToChar(ReadBytes(br, 2), 0);
        }

        public static byte ReadByteHelper(this BinaryReader br)
        {
            return br.ReadByte();
        }

        public static string ReadStringHelper(this BinaryReader br)
        {
            short length = br.ReadInt16Helper();
            return Encoding.UTF8.GetString(ReadBytes(br, length, false));
        }

        private static byte[] ReadBytes(BinaryReader br, int length, bool reverse = true)
        {
            byte[] buf = new byte[length];
            if (br.Read(buf, 0, buf.Length) != buf.Length)
            {
                throw new EndOfStreamException();
            }
            if (reverse)
            {
                Array.Reverse(buf);
            }
            return buf;
        }

        public static void WriteHelper(this BinaryWriter bw, byte value)
        {
            bw.Write(value);
        }

        public static void WriteHelper(this BinaryWriter bw, int value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Array.Reverse(bytes);
            bw.Write(bytes, 0, (int)bytes.Length);
        }

        public static void WriteHelper(this BinaryWriter bw, long value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Array.Reverse(bytes);
            bw.Write(bytes, 0, (int)bytes.Length);
        }

        public static void WriteHelper(this BinaryWriter bw, short value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Array.Reverse(bytes);
            bw.Write(bytes, 0, (int)bytes.Length);
        }

        public static void WriteHelper(this BinaryWriter bw, float value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Array.Reverse(bytes);
            bw.Write(bytes, 0, (int)bytes.Length);
        }

        public static void WriteHelper(this BinaryWriter bw, double value)
        {
            byte[] bytes = BitConverter.GetBytes(value);
            Array.Reverse(bytes);
            bw.Write(bytes, 0, (int)bytes.Length);
        }

        public static void WriteHelper(this BinaryWriter bw, string value)
        {
            if (!(Encoding.UTF8.GetByteCount(value) > 32767))
            {
                byte[] bytes = Encoding.UTF8.GetBytes(value);
                bw.WriteHelper((short)bytes.Length);
                bw.Write(bytes, 0, (int)bytes.Length);
            }
        }
    }
}
